<!doctype html>
<html>
    <head>
        <title>eID test</title>
        <style>
            body {
                text-align: center;
                font-family: sans-serif;
                }
        </style>
    </head>

    <br><br>
    <h2>HEY THERE, {{ id_auth }}!</h2>
    <pre>{{ request.user }}</pre>
</html>
